import os

def convert_path(in_path, out_dir, subdir=""):
    """
    Convertit un chemin de fichier

    Parameters
    ----------
    in_path: str
        chemin initial
    out_dir: str
        dossier de sortie
    """
    out_path = out_dir
    filename = os.path.basename(in_path)

    return os.path.join(out_path, subdir, filename)



from collections import defaultdict

def output_costs_from_costs_config(costs_config, row):
    """
    Fonction pour calculer les coûts associés à une arrête par rapport à la configuration des coûts

    Parameters
    ----------
    costs_config: dict
        défintion des coûts. objet correspondant au schéma JSON de configuration des coûts.
    row: dict
        ligne de la base de données correspondant à l'arrête sur laquelle calculer des coûts

    Returns
    -------
    tuple
        tuple des valeurs des coûts calculés pour row
    """

    values = {}
    output_costs = ()

    # Récupération des variables
    for variable in costs_config["variables"]:
        if variable["mapping"] == "value":
            values[variable["name"]] = row[variable["column_name"]]
        else:
            values[variable["name"]] = variable["mapping"][str(row[variable["column_name"]])]

    # Calcul des outputs
    for output in costs_config["outputs"]:
        result_direct = 0
        result_reverse = 0
        # Passage en defaultdict pour la gestion des champs facultatifs
        output = defaultdict(bool, output)
        if output["direct_conditions"]:
            if not _conditions_to_bool(output["direct_conditions"], values):
                result_direct = -1
        if output["reverse_conditions"]:
            if not _conditions_to_bool(output["reverse_conditions"], values):
                result_reverse = -1

        # Si les deux résultats sont à -1, inutile de faire l'opération
        if result_direct != -1 or result_reverse != -1:
            result = compute_operations(output["operations"], values)

            if result_direct != -1:
                result_direct = result

            if result_reverse != -1:
                result_reverse = result

        output_costs += (result_direct, result_reverse)

    return output_costs

def compute_operations(operations, values):
    """
    Calcule la valeur numérique du coût à partir des opérations définies dans la configuration

    Parameters
    ----------
    operations: array
        tableau comprenant des opérations, au format ["opération", valeur] avac valeur pouvant être
        dans values ou bien une valeur numérique
    values: dict
        Dictionnaire des valeurs de la configuration des coûts

    Returns
    -------
    float
        le résultat du calcul du coût
    """
    result = 0
    for operation in operations:
        if operation[0] == "add":
            if isinstance(operation[1], str):
                result += values[operation[1]]
            else:
                result += operation[1]
        elif operation[0] == "substract":
            if isinstance(operation[1], str):
                result -= values[operation[1]]
            else:
                result -= operation[1]
        elif operation[0] == "multiply":
            if isinstance(operation[1], str):
                result *= values[operation[1]]
            else:
                result *= operation[1]
        elif operation[0] == "divide":
            if isinstance(operation[1], str):
                result /= values[operation[1]]
            else:
                result /= operation[1]

    return result

def compute_operations_string(operations):
    """
    Génère la chaine de caractères correspondant au calcul des opérations
    Parameters
    ----------
    operations: array
        tableau comprenant des opérations, au format ["opération", valeur] avac valeur pouvant être
        Dictionnaire des valeurs de la configuration des coûts

    Returns
    -------
    string
        chaine de caractères correspondant au calcul des opérations
    """
    operation_str = "0";
    for operation in operations:
        if operation[0] == "add":
            operation_str += "+"
        elif operation[0] == "substract":
            operation_str += "-"
        elif operation[0] == "multiply":
            operation_str += "*"
        elif operation[0] == "divide":
            operation_str += "/"

        operation_str += str(operation[1])

    return operation_str


def _conditions_to_bool(conditions_str, values):
    """
    Fonction pour parser une chaîne de caractères correspondant à des conditions vers un booléen
    Se comporte comme un AND sur l'ensemble des conditions -> vrai si TOUT est vrai, false sinon

    Parameters
    ----------
    conditions_str: str
        chaine de caractère correspondant à toutes les conditions
    values: dict
        Dictionnaire des valeurs de la configuration des coûts

    Returns
    -------
    bool
        true si TOUTES les conditions sont vérifiée, false sinon
    """

    conditions = conditions_str.split(";")
    for condition in conditions:
        if not _condition_to_bool(condition, values):
            return False

    return True


def _condition_to_bool(condition, values):
    """
    Fonction pour parser une condition vers un booléen

    Parameters
    ----------
    condition: str
        chaine de caractère correspondant à 1 condition
    values: dict
        Dictionnaire des valeurs de la configuration des coûts

    Returns
    -------
    bool
        true si la condition est vérifiée, false sinon
    """

    # On teste 1 à 1 les différents tests possibles, en faisant bien attention à
    # tester ">=" avant ">" car le second est compris dans le premier (idem pour "<=")
    #
    # La conversion en float ne se fait qu'ici (dans les opérateurs > < >= <=) car on pourrait comparer des strings
    # avec == et !=
    test = condition.split("==")
    if len(test) == 2:
        return values[test[0]] == test[1].strip("'")

    test = condition.split("~=")
    if len(test) == 2:
        return values[test[0]] != test[1].strip("'")

    test = condition.split(">=")
    if len(test) == 2:
        return values[test[0]] >= float(test[1])

    test = condition.split("<=")
    if len(test) == 2:
        return values[test[0]] <= float(test[1])

    test = condition.split(">")
    if len(test) == 2:
        return values[test[0]] > float(test[1])

    test = condition.split("<")
    if len(test) == 2:
        return values[test[0]] < float(test[1])



import argparse
import json
import os
from pathlib import Path
from urllib import request

# 3rd party
import pytest
from unittest.mock import patch

from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT

from r2gg import cli

import psycopg2 as psycopg

cur_dir = Path(__file__).parent.absolute()

# ############################################################################
# ########## Classes #############
# ################################
HOST = os.environ.get("POSTGRES_HOST", "172.17.0.1")
PORT = os.environ.get("PORT", 5555)
DBNAME = os.environ.get("POSTGRES_DB", "ign")
USER = os.environ.get("POSTGRES_USER", "ign")
PASS = os.environ.get("POSTGRES_PASSWORD", "ign")

# Schemas are defined in input configuration .json files
INPUT_SCHEMA = "input"
OUTPUT_SCHEMA = "output"

TRONCON_ROUTE_URL = "https://storage.gra.cloud.ovh.net/v1/AUTH_366279ce616242ebb14161b7991a8461/road2/troncon_route_marseille10.sql"
NON_COMMUNICATION_URL = "https://storage.gra.cloud.ovh.net/v1/AUTH_366279ce616242ebb14161b7991a8461/road2/non_communication_marseille10.sql"

@pytest.fixture
def init_database(tmp_path) -> None:
    """Init database for test."""

    con = psycopg.connect(host=HOST, dbname=DBNAME, user=USER, password=PASS, port=PORT)
    con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
    con.cursor().execute(f"CREATE SCHEMA IF NOT EXISTS {INPUT_SCHEMA}")
    con.cursor().execute(f"CREATE DATABASE pivot")
    con.commit()
    con.close()

    # Insert test data
    con = psycopg.connect(host=HOST, dbname=DBNAME, user=USER, password=PASS, port=PORT)
    cur = con.cursor()
    cur.execute(f"SET search_path TO {INPUT_SCHEMA}")

    request.urlretrieve(TRONCON_ROUTE_URL, tmp_path / "troncon_route_marseille10.sql")
    request.urlretrieve(NON_COMMUNICATION_URL, tmp_path / "non_communication_marseille10.sql")

    with open(str(tmp_path / "troncon_route_marseille10.sql"), mode="r") as sql_script:
        cur.execute(sql_script.read())
    with open(str(tmp_path / "non_communication_marseille10.sql"), mode="r") as sql_script:
        cur.execute(sql_script.read())

    con.commit()
    con.close()

    # Add extensions to pivot
    con = psycopg.connect(host=HOST, dbname="pivot", user=USER, password=PASS, port=PORT)
    con.cursor().execute(f"CREATE SCHEMA IF NOT EXISTS {OUTPUT_SCHEMA}")
    con.cursor().execute("CREATE EXTENSION IF NOT EXISTS postgres_fdw")
    con.cursor().execute("CREATE EXTENSION IF NOT EXISTS Postgis")
    con.commit()
    con.close()
    yield

    con = psycopg.connect(host=HOST, dbname=DBNAME, user=USER, password=PASS, port=PORT)
    con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
    con.cursor().execute("DROP DATABASE pivot")
    con.cursor().execute(f"DROP TABLE {INPUT_SCHEMA}.non_communication")
    con.cursor().execute(f"DROP TABLE {INPUT_SCHEMA}.troncon_de_route")
    con.commit()
    con.close()


def update_src_dir_json(file_path: Path, output_file_path: Path):
    """Update .json file to replace {src_dir} by current source directory

    Parameters
    ----------
    file_path (Path) database .json file path
    """
    with open(file_path, mode="r") as f:
        content = f.read().replace("{src_dir}", str(Path(cur_dir / "..").resolve()))

    with open(output_file_path, mode="w") as f:
        f.write(content)


def sql2pivot():
    """Simple run of cli for pivot base creation."""
    # Update input json file to indicate current source directory
    update_src_dir_json(cur_dir / "config" / "sql2pivot.json", cur_dir / "config" / "updated_sql2pivot.json")

    # mock ArgumentParser for configuration file
    with patch("argparse.ArgumentParser.parse_args") as parse_arg_mock:
        parse_arg_mock.return_value = argparse.Namespace(
            config_file_path=str(cur_dir / "config" / "updated_sql2pivot.json"))
        # Run conversion
        cli.sql2pivot()


def test_sql2pivot(init_database):
    """Test simple run of cli for pivot base creation."""
    sql2pivot()


def pivot2osm():
    """Simple run of cli for osm file creation from pivot database."""
    # Update input json file to indicate current source directory
    update_src_dir_json(cur_dir / "config" / "pivot2osm.json", cur_dir / "config" / "updated_pivot2osm.json")

    # mock ArgumentParser for configuration file
    with patch("argparse.ArgumentParser.parse_args") as parse_arg_mock:
        parse_arg_mock.return_value = argparse.Namespace(
            config_file_path=str(cur_dir / "config" / "updated_pivot2osm.json"))
        # Run conversion
        cli.pivot2osm()

        assert os.path.exists("/home/docker/data/generation/pivot-osm.osm")


def test_pivot2osm(init_database):
    """Test simple run of cli for osm file creation from pivot database."""

    sql2pivot()
    pivot2osm()




import math
import time
import sys

from psycopg2.extras import DictCursor

from r2gg._output_costs_from_costs_config import output_costs_from_costs_config
from r2gg._read_config import config_from_path
from r2gg._sql_building import getQueryByTableAndBoundingBox

def pivot_to_pgr(source, cost_calculation_file_path, connection_work, connection_out, schema, input_schema, logger):
    """
    Fonction de conversion depuis la bdd pivot vers la base pgr

    Parameters
    ----------
    source: dict
    cost_calculation_file_path: str
        chemin vers le fichier json de configuration des coûts
    connection_work: psycopg2.connection
        connection à la bdd de travail
    connection_out: psycopg2.connection
        connection à la bdd pgrouting de sortie
    schema: str
        nom du schéma dans la base de sortie
    input_schema: str
        nom du schéma dans la base en entrée
    logger: logging.Logger
    """

    cursor_in = connection_work.cursor(cursor_factory=DictCursor, name="cursor_in")
    ways_table_name = schema + '.ways'
    # Récupération des coûts à calculer
    costs = config_from_path(cost_calculation_file_path)

    cursor_out = connection_out.cursor()
    # Création de la edge_table pgrouting
    create_table = """
        DROP TABLE IF EXISTS {0};
        CREATE TABLE {0}(
            id bigserial unique,
            tag_id integer,
            length double precision,
            length_m double precision,
            name text,
            source bigint,
            target bigint,
            x1 double precision,
            y1 double precision,
            x2 double precision,
            y2 double precision,
            importance double precision DEFAULT 6,
            the_geom geometry(Linestring,4326),
            cleabs text,
            nom_1_gauche text,
            nom_1_droite text,
            cpx_numero text,
            cpx_toponyme_route_nommee text,
            nature text,
            vitesse_moyenne_vl integer,
            position_par_rapport_au_sol integer,
            acces_vehicule_leger text,
            largeur_de_chaussee double precision,
            nombre_de_voies text,
            insee_commune_gauche text,
            insee_commune_droite text,
            itineraire_vert boolean,
            sens_de_circulation text,
            reserve_aux_bus text,
            urbain boolean,
            acces_pieton text,
            nature_de_la_restriction text,
            restriction_de_hauteur double precision,
            restriction_de_poids_total double precision,
            restriction_de_poids_par_essieu double precision,
            restriction_de_largeur double precision,
            restriction_de_longueur double precision,
            matieres_dangereuses_interdites boolean,
            cpx_gestionnaire text,
            cpx_numero_route_europeenne text,
            cpx_classement_administratif text,
            transport_exceptionnel boolean,
            vla_par_defaut integer,
            cout_penalites numeric,
            vehicule_leger_interdit boolean,
            cout_vehicule_prioritaire numeric
        );""".format(ways_table_name)
    logger.debug("SQL: {}".format(create_table))
    cursor_out.execute(create_table)

    # Ajout des colonnes de coûts
    add_columns = "ALTER TABLE {} ".format(ways_table_name)
    for output in costs["outputs"]:
        add_columns += "ADD COLUMN IF NOT EXISTS {} double precision,".format(output["name"])
        add_columns += "ADD COLUMN IF NOT EXISTS {} double precision,".format("reverse_" + output["name"])
    add_columns = add_columns[:-1]
    logger.debug("SQL: adding costs columns \n {}".format(add_columns))
    cursor_out.execute(add_columns)

    logger.info("Starting conversion")
    start_time = time.time()

    # Non communications ---------------------------------------------------------------------------
    logger.info("Writing turn restrinctions...")
    create_non_comm = """
        DROP TABLE IF EXISTS {0}.turn_restrictions;
        CREATE TABLE {0}.turn_restrictions(
            id bigserial unique,
            id_from bigint,
            id_to bigint
    );""".format(schema)
    logger.debug("SQL: {}".format(create_non_comm))
    cursor_out.execute(create_non_comm)

    logger.info("Populating turn restrictions")
    tr_query = f"SELECT id_from, id_to FROM {input_schema}.non_comm;"

    logger.debug("SQL: {}".format(tr_query))
    st_execute = time.time()
    cursor_in.execute(tr_query)
    et_execute = time.time()
    logger.info("Execution ended. Elapsed time : %s seconds." %(et_execute - st_execute))
    # Insertion petit à petit -> plus performant
    logger.info("SQL: Inserting or updating {} values in out db".format(cursor_in.rowcount))
    st_execute = time.time()
    index = 0
    batchsize = 10000
    rows = cursor_in.fetchmany(batchsize)
    while rows:
        values_str = ""
        for row in rows:
            values_str += "(%s, %s, %s),"
        values_str = values_str[:-1]

        # Tuple des valuers à insérer
        values_tuple = ()
        for row in rows:
            values_tuple += (index, row['id_from'], row['id_to'])
            index += 1

        set_on_conflict = (
            "id_from = excluded.id_from,id_to = excluded.id_to"
        )

        sql_insert = """
            INSERT INTO {}.turn_restrictions (id, id_from, id_to)
            VALUES {}
            ON CONFLICT (id) DO UPDATE
              SET {};
        """.format(schema, values_str, set_on_conflict)
        cursor_out.execute(sql_insert, values_tuple)
        connection_out.commit()
        rows = cursor_in.fetchmany(batchsize)

    et_execute = time.time()
    cursor_in.close()
    logger.info("Writing turn restrinctions Done. Elapsed time : %s seconds." %(et_execute - st_execute))

    # Noeuds ---------------------------------------------------------------------------------------
    logger.info("Writing vertices...")
    cursor_in = connection_work.cursor(cursor_factory=DictCursor, name="cursor_in")
    create_nodes = """
        DROP TABLE IF EXISTS {0}_vertices_pgr;
        CREATE TABLE {0}_vertices_pgr(
            id bigserial unique,
            cnt int,
            chk int,
            ein int,
            eout int,
            the_geom geometry(Point,4326)
    );""".format(ways_table_name)
    logger.debug("SQL: {}".format(create_nodes))
    cursor_out.execute(create_nodes)

    logger.info("Populating vertices")
    nd_query = f"SELECT id, geom FROM {input_schema}.nodes;"

    logger.debug("SQL: {}".format(nd_query))
    st_execute = time.time()
    cursor_in.execute(nd_query)
    et_execute = time.time()
    logger.info("Execution ended. Elapsed time : %s seconds." %(et_execute - st_execute))
    # Insertion petit à petit -> plus performant
    # logger.info("SQL: Inserting or updating {} values in out db".format(cursor_in.rowcount))
    st_execute = time.time()
    index = 0
    batchsize = 10000
    rows = cursor_in.fetchmany(batchsize)
    while rows:
        values_str = ""
        for row in rows:
            values_str += "(%s, %s),"
        values_str = values_str[:-1]

        # Tuple des valeurs à insérer
        values_tuple = ()
        for row in rows:
            values_tuple += (row['id'], row['geom'])
            index += 1

        set_on_conflict = (
            "the_geom = excluded.the_geom"
        )

        sql_insert = """
            INSERT INTO {}_vertices_pgr (id, the_geom)
            VALUES {}
            ON CONFLICT (id) DO UPDATE
              SET {};
        """.format(ways_table_name, values_str, set_on_conflict)
        cursor_out.execute(sql_insert, values_tuple)
        connection_out.commit()
        rows = cursor_in.fetchmany(batchsize)


    et_execute = time.time()
    cursor_in.close()
    logger.info("Writing vertices Done. Elapsed time : %s seconds." %(et_execute - st_execute))

    # Ways -----------------------------------------------------------------------------------------
    # Colonnes à lire dans la base source (champs classiques + champs servant aux coûts)
    cursor_in = connection_work.cursor(cursor_factory=DictCursor, name="cursor_in")
    attribute_columns = [
            'id',
            'geom as the_geom',
            'source_id as source',
            'target_id as target',
            'x1',
            'y1',
            'x2',
            'y2',
            'ST_Length(geom) as length',
            'length_m as length_m',
            'cleabs as cleabs',
            'importance as importance',
            'nature as nature',
            'nom_1_gauche as nom_1_gauche',
            'nom_1_droite as nom_1_droite',
            'cpx_numero as cpx_numero',
            'cpx_toponyme_route_nommee as cpx_toponyme_route_nommee',
            'vitesse_moyenne_vl as vitesse_moyenne_vl',
            'position_par_rapport_au_sol as position_par_rapport_au_sol',
            'acces_vehicule_leger as acces_vehicule_leger',
            'largeur_de_chaussee as largeur_de_chaussee',
            'nombre_de_voies as nombre_de_voies',
            'insee_commune_gauche as insee_commune_gauche',
            'insee_commune_droite as insee_commune_droite',
            'itineraire_vert as itineraire_vert',
            'sens_de_circulation as sens_de_circulation',
            'reserve_aux_bus as reserve_aux_bus',
            'urbain as urbain',
            'acces_pieton as acces_pieton',
            'nature_de_la_restriction as nature_de_la_restriction',
            'restriction_de_hauteur as restriction_de_hauteur',
            'restriction_de_poids_total as restriction_de_poids_total',
            'restriction_de_poids_par_essieu as restriction_de_poids_par_essieu',
            'restriction_de_largeur as restriction_de_largeur',
            'restriction_de_longueur as restriction_de_longueur',
            'matieres_dangereuses_interdites as matieres_dangereuses_interdites',
            'cpx_gestionnaire as cpx_gestionnaire',
            'cpx_numero_route_europeenne as cpx_numero_route_europeenne',
            'cpx_classement_administratif as cpx_classement_administratif',
            'transport_exceptionnel as transport_exceptionnel',
            'vla_par_defaut as vla_par_defaut',
            'cout_penalites as cout_penalites',
            'vehicule_leger_interdit as vehicule_leger_interdit',
            'cout_vehicule_prioritaire as cout_vehicule_prioritaire'
        ]
    in_columns = attribute_columns.copy()
    for variable in costs["variables"]:
        in_columns += [variable["column_name"]]

    output_columns_names = [column.split(' ')[-1] for column in attribute_columns]

    # Ecriture des ways
    sql_query = getQueryByTableAndBoundingBox(f'{input_schema}.edges', source['bbox'], in_columns)
    logger.info("SQL: {}".format(sql_query))
    st_execute = time.time()
    cursor_in.execute(sql_query)
    et_execute = time.time()
    logger.info("Execution ended. Elapsed time : %s seconds." %(et_execute - st_execute))

    # Chaîne de n %s, pour l'insertion de données via psycopg
    single_value_str = "%s," * (len(attribute_columns) + 2 * len(costs["outputs"]))
    single_value_str = single_value_str[:-1]

    # Insertion petit à petit -> plus performant
    # logger.info("SQL: Inserting or updating {} values in out db".format(cursor_in.rowcount))
    st_execute = time.time()
    batchsize = 10000
    percent = 0
    rows = cursor_in.fetchmany(batchsize)
    while rows:
        percent += 1000000 / cursor_in.rowcount
        # Chaîne permettant l'insertion de valeurs via psycopg
        values_str = ""
        for row in rows:
            values_str += "(" + single_value_str + "),"
        values_str = values_str[:-1]

        # Tuple des valuers à insérer
        values_tuple = ()
        for row in rows:
            output_costs = output_costs_from_costs_config(costs, row)
            values_tuple += tuple(
                row[ output_columns_name ] for output_columns_name in output_columns_names
            ) + output_costs

        output_columns = "("
        for output_columns_name in output_columns_names:
            output_columns += output_columns_name + ','
        output_columns = output_columns[:-1]

        set_on_conflict = ''
        for output_columns_name in output_columns_names:
            set_on_conflict += "{0} = excluded.{0},".format(output_columns_name)
        set_on_conflict = set_on_conflict[:-1]

        for output in costs["outputs"]:
            output_columns += "," + output["name"] + ",reverse_" + output["name"]
            set_on_conflict += ",{0} = excluded.{0}".format(output["name"])
            set_on_conflict += ",{0} = excluded.{0}".format("reverse_" + output["name"])

        output_columns += ")"
        sql_insert = """
            INSERT INTO {} {}
            VALUES {}
            ON CONFLICT (id) DO UPDATE
              SET {};
            """.format(ways_table_name, output_columns, values_str, set_on_conflict)
        cursor_out.execute(sql_insert, values_tuple)
        connection_out.commit()
        rows = cursor_in.fetchmany(batchsize)

    et_execute = time.time()
    cursor_in.close();
    logger.info("Writing ways ended. Elapsed time : %s seconds." %(et_execute - st_execute))

    spacial_indices_query = """
        CREATE INDEX IF NOT EXISTS ways_geom_gist ON {0} USING GIST (the_geom);
        CREATE INDEX IF NOT EXISTS ways_vertices_geom_gist ON {0}_vertices_pgr USING GIST (the_geom);
        CLUSTER {0} USING ways_geom_gist ;
        CLUSTER {0}_vertices_pgr USING ways_vertices_geom_gist ;
        CREATE INDEX IF NOT EXISTS ways_importance_idx ON {0} USING btree (importance);
    """.format(ways_table_name)
    logger.info("SQL: {}".format(spacial_indices_query))
    st_execute = time.time()
    cursor_out.execute(spacial_indices_query)
    et_execute = time.time()
    logger.info("Execution ended. Elapsed time : %s seconds." %(et_execute - st_execute))
    connection_out.commit()

    turn_restrictions_indices_query = """
        CREATE INDEX IF NOT EXISTS turn_restrictions_id_key ON {0}.turn_restrictions USING btree (id);
        CREATE INDEX IF NOT EXISTS ways_id_key ON {1} USING btree (id);
        CREATE INDEX IF NOT EXISTS ways_vertices_pgr_id_key ON {1}_vertices_pgr USING btree (id);
    """.format(schema, ways_table_name)
    logger.info("SQL: {}".format(turn_restrictions_indices_query))
    st_execute = time.time()
    cursor_out.execute(turn_restrictions_indices_query)
    et_execute = time.time()
    logger.info("Execution ended. Elapsed time : %s seconds." %(et_execute - st_execute))
    connection_out.commit()

    old_isolation_level = connection_out.isolation_level
    connection_out.set_isolation_level(0)

    # VACCUM ANALYZE for ways
    vacuum_query = f"VACUUM ANALYZE {ways_table_name};"
    logger.info("SQL: {}".format(vacuum_query))
    st_execute = time.time()
    cursor_out.execute(vacuum_query)
    et_execute = time.time()
    logger.info("Execution ended. Elapsed time : %s seconds." %(et_execute - st_execute))

    # VACCUM ANALYZE for ways_vertices_pgr
    vacuum_query = f"VACUUM ANALYZE {ways_table_name}_vertices_pgr;"
    logger.info("SQL: {}".format(vacuum_query))
    st_execute = time.time()
    cursor_out.execute(vacuum_query)
    et_execute = time.time()
    logger.info("Execution ended. Elapsed time : %s seconds." %(et_execute - st_execute))

    # VACCUM ANALYZE for turn_restrictions
    vacuum_query = f"VACUUM ANALYZE {schema}.turn_restrictions;"
    logger.info("SQL: {}".format(vacuum_query))
    st_execute = time.time()
    cursor_out.execute(vacuum_query)
    et_execute = time.time()
    logger.info("Execution ended. Elapsed time : %s seconds." %(et_execute - st_execute))

    connection_out.set_isolation_level(old_isolation_level)
    connection_out.commit()

    cursor_out.close()

    # Nettoyage du graphe
    logger.info("Cleaning isolated clusters of less than 10 edges...")
    cursor_isolated = connection_out.cursor()

    profile_names = set([ cost['profile'] for cost in source["costs"]])
    st_execute = time.time()

    for profile_name in profile_names:
        logger.info("Cleaning isolated edges for profile {}...".format(profile_name))
        clean_graph_query = """
        WITH connected_components AS (
          SELECT * FROM pgr_connectedComponents(
            'SELECT id, source, target, cost_s_{1} as cost, reverse_cost_s_{1} as reverse_cost FROM {0}.ways'
          )
        ),
        remove_nodes AS (
          SELECT node
          FROM
            connected_components
          WHERE
          component = ANY(
            SELECT DISTINCT component
            FROM
            (
              SELECT component, count(*) AS nb
              FROM
                connected_components
              GROUP BY component
            )
            AS components
            WHERE nb <= 10
          )
        )
        UPDATE {0}.ways
        SET cost_s_{1} = -1,
          reverse_cost_s_{1} = -1,
          cost_m_{1} = -1,
          reverse_cost_m_{1} = -1
        WHERE {0}.ways.target = ANY(SELECT * from remove_nodes) OR {0}.ways.source = ANY(SELECT * from remove_nodes);
        """.format(schema, profile_name)
        logger.info("SQL: {}".format(clean_graph_query))
        cursor_isolated.execute(clean_graph_query)
        connection_out.commit()

    et_execute = time.time()
    logger.info("Execution ended. Elapsed time : %s seconds." %(et_execute - st_execute))
    cursor_isolated.close()

    end_time = time.time()
    logger.info("Conversion from pivot to PGR ended. Elapsed time : %s seconds." %(end_time - start_time))



def getQueryByTableAndBoundingBox(table, bbox, columns=['*'], whereClauses=None):
    """
    Construit une requête SQL spatiale en fonction d'une BBOX au format string:
    xmin,ymin,xmax,ymax

    Parameters
    ----------
    table: str
    bbox: str
        'xmin,ymin,xmax,ymax'
    columns: [str]
        colonnes à selectionner
        par défaut toutes : ['*']
    whereClauses: [str]
        conditions qui ne sont pas liées à la bbox

    Returns
    -------
    dict
        configuration normalisée
    """
    if whereClauses is None:
        whereClauses=['true']
    if bbox:
        whereClauses.append(
            'geom && ST_MakeEnvelope(%s)' % bbox
        )

    sql  = 'SELECT %s FROM %s ' %(','.join(columns), table)
    sql += 'WHERE ' + ' AND '.join(whereClauses)
    sql += ' ORDER BY id ASC'
    return sql



#! python3  # noqa: E265

"""Main CLI entrypoint."""

# Package
from r2gg.__about__ import (
    __author__,
    __cli_usage__,
    __summary__,
    __title__,
    __title_clean__,
    __uri_homepage__,
    __version__,
)
from r2gg._configure import configure, connect_working_db, disconnect_working_db
from r2gg._main import sql_convert, pgr_convert, osm_convert, osrm_convert, valhalla_convert, write_road2_config

# ############################################################################
# ########## MAIN ################
# ################################
def sql2pivot():
    config, resource, db_configs, logger = configure()
    connection = connect_working_db(config, db_configs, logger)
    sql_convert(config, resource, db_configs, connection, logger)
    disconnect_working_db(connection, logger)

def pivot2pgrouting():
    config, resource, db_configs, logger = configure()
    connection = connect_working_db(config, db_configs, logger)
    pgr_convert(config, resource, db_configs, connection, logger)
    disconnect_working_db(connection, logger)

def pivot2osm():
    config, resource, db_configs, logger = configure()
    connection = connect_working_db(config, db_configs, logger)
    osm_convert(config, resource, db_configs, connection, logger)
    disconnect_working_db(connection, logger)

def osm2osrm():
    config, resource, _, logger = configure()
    osrm_convert(config, resource, logger)

def osm2valhalla():
    config, resource, _, logger = configure()
    valhalla_convert(config, resource, logger)

def road2config():
    config, resource, _, logger = configure()
    write_road2_config(config, resource, logger)

def main():
    """Main CLI entrypoint.
    """
    config, resource, db_configs, logger = configure()
    connection = connect_working_db(config, db_configs, logger)
    sql_convert(config, resource, db_configs, connection, logger)
    if (resource['type'] in ['pgr', 'smartpgr']):
        config, resource, db_configs, connection, logger = configure()
        pgr_convert(config, resource, db_configs, connection, logger)
        disconnect_working_db(connection, logger)
    elif (resource['type'] == 'osrm'):
        config, resource, db_configs, connection, logger = configure()
        osm_convert(config, resource, db_configs,  connection, logger)
        disconnect_working_db(connection, logger)
        osrm_convert(config, resource, logger)
    elif (resource['type'] == 'valhalla'):
        config, resource, db_configs, connection, logger = configure()
        osm_convert(config, resource, db_configs, connection, logger, True)
        valhalla_convert(config, resource, logger)
    else:
        raise ValueError("Wrong resource type, should be in ['pgr',osrm','valhalla','smartpgr']")
    write_road2_config(config, resource, logger)

# -- Stand alone execution
if __name__ == "__main__":
    main()  # required by unittest



import osmium
import os
import time

class _PBFWriter(osmium.SimpleHandler):
    def __init__(self, writer):
        osmium.SimpleHandler.__init__(self)
        self.writer = writer
    def node(self, n):
        self.writer.add_node(n)
    def way(self, w):
        self.writer.add_way(w)
    def relation(self, r):
        self.writer.add_relation(r)

def osm_to_pbf(in_path, out_path, logger):
    """
    Fonction de conversion depuis un fichier .osm vers un fichier .osm.pbf

    Parameters
    ----------
    in_path: str
        chemin du fichier .osm en entrée
    out_path: str
        chemin du fichier .osm.pbf en entrée
    logger: logging.Logger
    """

    logger.info("Writing pbf file: {} from osm file {}".format(out_path, in_path))
    start_time = time.time()
    if os.path.exists(out_path):
      os.remove(out_path)
    writer = osmium.SimpleWriter(out_path)
    handler = _PBFWriter(writer)
    handler.apply_file(in_path)
    os.remove(in_path)
    end_time = time.time()
    logger.info("PBF writing done. Elapsed time : %s seconds." %(end_time - start_time))



import json
import multiprocessing
import os
import json
import time
from datetime import datetime

import psycopg2
# https://github.com/andialbrecht/sqlparse
import sqlparse

from r2gg._lua_builder import build_lua
from r2gg._pivot_to_osm import pivot_to_osm
from r2gg._pivot_to_pgr import pivot_to_pgr
from r2gg._read_config import config_from_path
from r2gg._subprocess_execution import subprocess_execution
from r2gg._path_converter import convert_path
from r2gg._file_copier import copy_file_locally
from r2gg._valhalla_lua_builder import build_valhalla_lua
from r2gg._osm_to_pbf import osm_to_pbf


def sql_convert(config, resource, db_configs, connection, logger):
    """
    Fonction de conversion depuis la bdd source vers la bdd pivot

    Parameters
    ----------
    config: dict
        dictionnaire correspondant à la configuration décrite dans le fichier passé en argument
    resource: dict
        dictionnaire correspondant à la resource décrite dans le fichier passé en argument
    db_configs: dict
        dictionnaire correspondant aux configurations des bdd
    connection: psycopg2.connection
        connection à la bdd de travail
    logger: logging.Logger
    """

    logger.info("Conversion from BDD to pivot")

    # Date de l'extraction pour la noter dans la configuration de la ressource
    extraction_date = datetime.now()
    # Ecriture dans un fichier temporaire de la date d'extraction
    work_dir_config = config['workingSpace']['directory']
    date_file = os.path.join(work_dir_config, "r2gg.date")
    date_time = extraction_date.strftime("%Y-%m-%d")
    logger.info("extraction date to save in " + date_file + ": " + date_time)

    f = open(date_file, "w")
    f.write(date_time)
    f.close()

    # Mesure du temps de l'ensemble des génération pivot
    st_sql_conversion = time.time()

    used_bases = []

    # Il y a potentiellement une conversion par source indiquée dans la ressource
    for source in resource[ 'sources' ]:

        logger.info("Create pivot of source: " + source['id'])

        # Les sources smartrouting n'ont pas besoin de génération mais peuvent apparaître dans certaines ressources
        if source['type'] == 'smartrouting':
            logger.info("Smartrouting source, no need for pivot")
            continue

        # Plusieurs sources peuvent référencer le même mapping mais changer plus tard dans la génération
        found_base = False
        for ub in used_bases:
            if ub == source['mapping']['source']['baseId']:
                found_base = True
        if found_base:
            logger.info("Mapping already done, create next source...")
            continue
        else:
            logger.info("Mapping not done")

        # Configuration de la bdd source
        source_db_config = db_configs[ source['mapping']['source']['baseId'] ]
        used_bases.append(source['mapping']['source']['baseId'])

        # Configuration de la bdd de travail utilisée pour ce pivot
        work_db_config = db_configs[ config['workingSpace']['baseId'] ]

        # Récupération de la bbox
        bbox = [float(coord) for coord in source["bbox"].split(",")]
        assert len(bbox) == 4, "bondingBox invalide"
        xmin = bbox[0]
        ymin = bbox[1]
        xmax = bbox[2]
        ymax = bbox[3]
        logger.info("Create source on bbox: " + source["bbox"])

        # Lancement du script SQL de conversion source --> pivot
        connection.autocommit = True
        with open( source['mapping']['conversion']['file'] ) as sql_script:
            cur = connection.cursor()
            logger.info("Executing SQL conversion script")
            instructions = sqlparse.split(sql_script.read().format(user=work_db_config.get('user'),
                                                                   input_schema=source_db_config.get('schema'),
                                                                   output_schema=work_db_config.get('schema')
                                                                   ))

            # Exécution instruction par instruction
            for instruction in instructions:
                if instruction == '':
                    continue
                logger.debug("SQL:\n{}\n".format(instruction) )
                st_instruction = time.time()
                cur.execute(instruction,
                    {
                    'bdpwd': source_db_config.get('password'), 'bdport': source_db_config.get('port'),
                    'bdhost': source_db_config.get('host'), 'bduser': source_db_config.get('user'),
                    'dbname': source_db_config.get('database'),
                    'xmin': xmin, 'ymin': ymin, 'xmax': xmax, 'ymax': ymax
                    }
                )
                et_instruction = time.time()
                logger.info("Execution ended. Elapsed time : %s seconds." %(et_instruction - st_instruction))

    et_sql_conversion = time.time()

    logger.info("Conversion from BDD to pivot ended. Elapsed time : %s seconds." %(et_sql_conversion - st_sql_conversion))

def pgr_convert(config, resource, db_configs, connection, logger):
    """
    Fonction de conversion depuis la bdd pivot vers la bdd pgrouting

    Parameters
    ----------
    config: dict
        dictionnaire correspondant à la configuration décrite dans le fichier passé en argument
    resource: dict
        dictionnaire correspondant à la resource décrite dans le fichier passé en argument
    db_configs: dict
        dictionnaire correspondant aux configurations des bdd
    connection: psycopg2.connection
        connection à la bdd de travail
    logger: logging.Logger
    """

    if (resource['type'] not in ['pgr', 'smartpgr']):
        raise ValueError("Wrong resource type, should be 'pgr' or 'smartpgr'")

    logger.info("Conversion from pivot to PGR")
    st_pivot_to_pgr = time.time()

    i = 0
    for source in resource["sources"]:

        logger.info("Source {} of {}...".format(i+1, len(resource["sources"])))
        logger.info("Source id : " + source["id"])

        # Configuration et connection à la base de sortie
        out_db_config = db_configs[ source['storage']['base']['baseId'] ]
        host = out_db_config.get('host')
        dbname = out_db_config.get('database')
        user = out_db_config.get('user')
        password = out_db_config.get('password')
        port = out_db_config.get('port')
        connect_args = 'host=%s dbname=%s user=%s password=%s port=%s' %(host, dbname, user, password, port)
        logger.info("Connecting to output database")
        connection_out = psycopg2.connect(connect_args)

        schema_out = out_db_config.get('schema')

        source_db_config = db_configs[source['mapping']['source']['baseId']]
        input_schema = source_db_config.get('schema')

        cost_calculation_files_paths = {cost["compute"]["configuration"]["storage"]["file"] for cost in source["costs"]}

        for cost_calculation_file_path in cost_calculation_files_paths:
            pivot_to_pgr(source, cost_calculation_file_path, connection, connection_out, schema_out, input_schema, logger)
        connection_out.close()

    et_pivot_to_pgr = time.time()
    logger.info("Conversion from pivot to PGR ended. Elapsed time : %s seconds." %(et_pivot_to_pgr - st_pivot_to_pgr))


def osm_convert(config, resource, db_configs, connection, logger):
    """
    Fonction de conversion depuis la bdd pivot vers un fichier osm

    Parameters
    ----------
    config: dict
        dictionnaire correspondant à la configuration décrite dans le fichier passé en argument
    resource: dict
        dictionnaire correspondant à la resource décrite dans le fichier passé en argument
    db_configs: dict
        dictionnaire correspondant aux configurations des bdd
    connection: psycopg2.connection
        connection à la bdd de travail
    logger: logging.Logger
    """

    logger.info("Conversion from pivot to OSM format for a resource")

    used_bases = {}
    work_dir_config = config['workingSpace']['directory']

    # On vérifie le type de la ressource
    if (resource['type'] not in ['osrm', 'valhalla']):
        raise ValueError("Wrong resource type, should be in ['osrm','valhalla']")

    # Les outils de conversion utilisés par Valhalla ne lisent que du osm.pbf
    convert_osm_to_pbf = False
    if (resource['type'] == 'valhalla'):
        convert_osm_to_pbf = True

    # Comme chaque source de la ressource peut potentiellement nécessiter un pivot différent,
    # On fait une boucle sur les sources et on adapte en fonction du type
    for source in resource['sources']:

        logger.info("Create osm file of source: " + source['id'])

        # Les sources smartrouting n'ont pas besoin de génération mais peuvent apparaître dans certaines ressources
        if source['type'] == 'smartrouting':
            logger.info("Smartrouting source, no need for osm file")
            continue

        # Plusieurs sources peuvent référencer le même mapping mais changer plus tard dans la génération
        found_base = False
        found_id = ''
        for sid,sub in used_bases.items():
            if sub == source['mapping']['source']['baseId']:
                found_base = True
                found_id = sid

        if found_base:

            if convert_osm_to_pbf:
                linked_file = os.path.join(work_dir_config, source['id'] + ".osm.pbf")
                real_file = os.path.join(work_dir_config, found_id + ".osm.pbf")
            else:
                linked_file = os.path.join(work_dir_config, source['id'] + ".osm")
                real_file = os.path.join(work_dir_config, found_id + ".osm")

            logger.info("Mapping already done, creating a linked osm file : " + linked_file + " -> " + real_file)
            if not os.path.islink(linked_file):
                os.symlink(real_file, linked_file)
            else:
                ex_link = os.readlink(linked_file)
                ex_link = os.path.join(os.path.dirname(linked_file), ex_link)
                if ex_link != real_file:
                    raise ValueError("SymLink is already pointing to another file")
                else:
                    logger.info("SymLink already pointing to the good file")

        else:
            logger.info("Mapping not already done")
            pivot_to_osm(config, source, db_configs, connection, logger, convert_osm_to_pbf)

        used_bases[ source['id'] ] = source['mapping']['source']['baseId']

def osrm_convert(config, resource, logger, build_lua_from_cost_config = True):
    """
    Fonction de conversion depuis le fichier osm vers les fichiers osrm

    Parameters
    ----------
    config: dict
        dictionnaire correspondant à la configuration décrite dans le fichier passé en argument
    resource: dict
        dictionnaire correspondant à la resource décrite dans le fichier passé en argument
    logger: logging.Logger
    build_lua_from_cost_config : bool
        contruire le lua à partir de le configuration des coûts. Défaut : True
    """

    if (resource['type'] != 'osrm'):
        raise ValueError("Wrong resource type, should be 'osrm'")

    logger.info("Conversion from OSM to OSRM for a resource")

    logger.info("Generating graphs for each cost...")
    cpu_count = multiprocessing.cpu_count()
    work_dir_config = config['workingSpace']['directory']

    i = 0
    for source in resource["sources"]:

        logger.info("Source {} of {}...".format(i+1, len(resource["sources"])))

        logger.info('LUA part')
        lua_file = source["cost"]["compute"]["storage"]["file"]

        if build_lua_from_cost_config:
            logger.info("Building lua profile")
            config_file = source["cost"]["compute"]["configuration"]["storage"]["file"]
            costs_config = config_from_path(config_file)
            cost_name = source["cost"]["compute"]["configuration"]["name"]

            if cost_name not in [ output["name"] for output in costs_config["outputs"] ]:
                raise ValueError("cost_name must be in cost configuration")

            with open(lua_file, "w") as lua_f:
                lua_f.write(build_lua(costs_config, cost_name))
            logger.info("Finished lua building")

        logger.info('Recherche du fichier OSM')

        osm_file = os.path.join(work_dir_config, source['id'] + ".osm")
        extension = ".osm"
        if not os.path.exists(osm_file):
            osm_pbf_file = os.path.join(work_dir_config, source['id'] + ".osm.pbf")
            if not os.path.exists(osm_pbf_file):
                raise ValueError("Can't find osm file")
            else:
                osm_file = osm_pbf_file
                extension = ".osm.pbf"

        logger.info('Fichier OSM trouvé: ' + osm_file)

        # Gestion des points "." dans le chemin d'accès avec ".".join()
        osrm_file = source["storage"]["file"]
        cost_dir = os.path.dirname(osrm_file)
        profile_name = osrm_file.split("/")[-1].split(".")[0]
        tmp_osm_file = "{}/{}{}".format(cost_dir, profile_name, extension)

        # Définition des commandes shell à exécuter
        mkdir_args = ["mkdir", "-p", cost_dir]
        copy_args = ["cp", ".".join(osm_file.split(".")[:-1]) + extension, tmp_osm_file]
        osrm_extract_args = ["osrm-extract", tmp_osm_file, "-p", lua_file, "-t", cpu_count]
        osrm_contract_args = ["osrm-contract", osrm_file, "-t", cpu_count]
        rm_args = ["rm", tmp_osm_file]

        subprocess_execution(mkdir_args, logger)
        subprocess_execution(copy_args, logger)
        start_command = time.time()
        subprocess_execution(osrm_extract_args, logger)
        end_command = time.time()
        logger.info("OSRM extract ended. Elapsed time : %s seconds." %(end_command - start_command))
        subprocess_execution(osrm_contract_args, logger)
        final_command = time.time()
        logger.info("OSRM contract ended. Elapsed time : %s seconds." %(final_command - end_command))
        subprocess_execution(rm_args, logger)
        i += 1


def valhalla_convert(config, resource, logger, build_lua_from_cost_config = True):
    """
    Fonction de conversion depuis le fichier .osm.pbf vers les fichiers valhalla

    Parameters
    ----------
    config: dict
        dictionnaire correspondant à la configuration décrite dans le fichier passé en argument
    resource: dict
        dictionnaire correspondant à la resource décrite dans le fichier passé en argument
    logger: logging.Logger
    build_lua_from_cost_config : bool
        contruire le lua à partir de le configuration des coûts. Défaut : True
    """

    if (resource['type'] != 'valhalla'):
        raise ValueError("Wrong resource type, should be 'valhalla'")

    logger.info("Conversion from OSM PBF to VALHALLA for a resource")

    logger.info("Generating graphs for each set of tiles")

    work_dir_config = config['workingSpace']['directory']

    i = 0
    for source in resource["sources"]:

        logger.info("Source {} of {}...".format(i+1, len(resource["sources"])))

        logger.info('Looking for OSM PBF file')

        osm_file = os.path.join(work_dir_config, source['id'] + ".osm.pbf")
        extension = ".osm.pbf"
        if not os.path.exists(osm_file):
            # On gère le cas où une génération a déjà été lancée pour du OSRM donc en .osm et pas forcément pbf
            osm_tmp_file = os.path.join(work_dir_config, source['id'] + ".osm")
            if not os.path.exists(osm_tmp_file):
                raise ValueError("Can't find osm file")
            else:
                logger.info("OSM file found, conversion to pbf")
                osm_to_pbf(osm_tmp_file, osm_file, logger)

        logger.info('OSM PBF file found: ' + osm_file)

        # Todo : modifier toute cette partie LUA
        # actuellement, on génère un seul LUA pour tous les coûts possibles (car/fastest, car/shortest, pedestrian/shortest)
        # Mais il serait bien de générer le LUA avec les coûts qui sont dans costs uniquement
        logger.info('LUA part')

        lua_file = source["costs"][0]["compute"]["storage"]["file"]

        if build_lua_from_cost_config:
            logger.info("Building lua profile")
            config_file = source["costs"][0]["compute"]["configuration"]["storage"]["file"]
            costs_config = config_from_path(config_file)

            with open(lua_file, "w") as lua_f:
                lua_f.write(build_valhalla_lua(costs_config))
            logger.info("Finished lua building")

        # Définition et exécution des commandes shell à exécuter
        mkdir_args = ["mkdir", "-p", source["storage"]["dir"]]
        subprocess_execution(mkdir_args, logger)

        start_command = time.time()
        valhalla_build_config_args = ["valhalla_build_config",
            "--mjolnir-tile-dir",  source["storage"]["dir"],
            "--mjolnir-tile-extract", source["storage"]["tar"],
            # Modification des limites par défaut du service : 10h pour isochrone et 1000km pour iso distance
            # contre 2h et 200km par défaut
            "--service-limits-isochrone-max-time-contour", "600",
            "--service-limits-isochrone-max-distance-contour", "1000",
            # Ajout de l'autorisation à exclure les ponts/tunnels/péages
            "--service-limits-allow-hard-exclusions", "True"]
        subprocess_execution(valhalla_build_config_args, logger, outfile = source["storage"]["config"])
        # Nécessaire le temps que le fichier s'écrive...
        time.sleep(1)
        # Ajout du graph custom dans la config valhalla (impossible via les paramètres du build_config)
        with open(source["storage"]["config"], "r") as valhalla_config:
            config_dict = json.load(valhalla_config)
            config_dict["mjolnir"]["graph_lua_name"] = source["costs"][0]["compute"]["storage"]["file"]

        with open(source["storage"]["config"], "w") as valhalla_config:
            valhalla_config.write(json.dumps(config_dict))

        valhalla_build_tiles_args = ["valhalla_build_tiles", "-c", source["storage"]["config"], osm_file]
        subprocess_execution(valhalla_build_tiles_args, logger)

        valhalla_build_extract_args = ["valhalla_build_extract", "-c", source["storage"]["config"], "-v"]
        subprocess_execution(valhalla_build_extract_args, logger)

        final_command = time.time()
        logger.info("Valhalla tiles built. Elapsed time : %s seconds." %(final_command - start_command))


def write_road2_config(config, resource, logger, convert_file_paths = True):
    """
    Fonction pour l'écriture du fichier de ressource

    Parameters
    ----------
    config: dict
        dictionnaire correspondant à la configuration décrite dans le fichier passé en argument
    resource: dict
        dictionnaire correspondant à la resource décrite dans le fichier passé en argument
    logger: logging.Logger
    """

    logger.info("Writing sources")

    os.makedirs(config["outputs"]["configurations"]["sources"]["storage"]["directory"] or '.', exist_ok=True)

    source_ids = []

    for source in resource["sources"]:

        source_file = os.path.join(config["outputs"]["configurations"]["sources"]["storage"]["directory"], source['id'] + ".source")
        logger.info("Writing source file : " + source_file)

        # On modifie la source en fonction de son type
        if source['type'] == 'smartrouting':
            logger.info("Smartrouting source, no need for modifications")
        elif source['type'] == 'osrm':
            source.pop("mapping", None)
            source["cost"].pop("compute", None)
        elif source['type'] == "valhalla":
            source.pop("mapping", None)
            for cost in source["costs"]:
                cost.pop("compute", None)
        elif source['type'] == "pgr":
            source.pop("mapping", None)
            bid_tmp = source["storage"]["base"]["baseId"]
            for base in config["bases"]:
                if base["id"] == bid_tmp:
                    db_file_out = convert_path(base["configFile"], config["outputs"]["configurations"]["databases"]["storage"]["directory"])
                    copy_file_locally(base["configFile"], db_file_out)
                    source["storage"]["base"].update({"dbConfig":db_file_out})
                    source["storage"]["base"].update({"schema":base["schema"]})
            source["storage"]["base"].pop("baseId", None)
            for cost in source["costs"]:
                cost.pop("compute", None)
        else:
            continue

        # Écriture du fichier
        with open(source_file, "w") as source_file:
            json_string = json.dumps(source, indent=2)
            source_file.write(json_string)

        source_ids.append(source['id'])

    # On passe à la ressource
    resource_file = os.path.join(config["outputs"]["configurations"]["resource"]["storage"]["directory"], resource['id'] + ".resource")
    logger.info("Writing resource file: " + resource_file)

    # Récupération de la date d'extraction
    work_dir_config = config['workingSpace']['directory']
    date_file = os.path.join(work_dir_config, "r2gg.date")
    f = open(date_file, "r")
    extraction_date = f.read()
    logger.info("extraction date to add in resource (from "+ date_file +"): " + extraction_date)
    f.close()

    # On fait le dossier s'il n'existe pas
    os.makedirs(config["outputs"]["configurations"]["resource"]["storage"]["directory"] or '.', exist_ok=True)
    # On modifie l'objet resource
    resource['sources'] = source_ids
    resource["resourceVersion"] = extraction_date

    final_resource = {"resource": resource}
    with open(resource_file, "w") as resource_file:
        json_string = json.dumps(final_resource, indent=2)
        resource_file.write(json_string)
